@@ -166,7 +166,7 @@ class VocabularyResolver(object):
     def param_group_reference(self, id):
         return ParamGroupReference(id)
 
-    def param(self, name, value=None, cv_ref=None, **kwargs):
+    def param(self, name, value=None, cv_ref=None, **kwargs) -> Union[CVParam, UserParam, ParamGroupReference]:
         accession = kwargs.get("accession")
         if isinstance(name, CVParam):
             return name
@@ -422,6 +422,9 @@ class ReprBorrowingPartial(partial):
     def ensure_all(self, objs):
         return [self.ensure(obj or {}) for obj in ensure_iterable(objs)]
 
+    def register(self, identifier):
+        return self.kwargs['context'].register(self._func.__name__, identifier)
+
 
 class CallbackBindingPartial(ReprBorrowingPartial):
 
